; The following was originally Identify Cpu by MichaelW

;  98% is unchanged from Michael's version.  I (jimg), with apologies, have made changes so
;       that it copies the output to a buffer rather than to stdout using the print macro,
;       and also made changes so that macros.asm is not required (changed switch to .if).

    ; -------------------------------------------
    ; Returns a bit range value, with the value shifted to place lowbit at bit position 0.
    ; -------------------------------------------
    brval MACRO dwordval, highbit, lowbit
        mov   eax, dwordval
        shl   eax, 31-highbit
        shr   eax, 31-highbit+lowbit
        EXITM <eax>
    ENDM

    ; --------------------------------------------
    ; This will overwrite EAX, EBX, ECX, and EDX.
    ; --------------------------------------------
    _cpuid MACRO function
        mov   eax, function
        cpuid
    ENDM

    ; --------------------------------------
    ; Returns 1 if the specified feature is supported, otherwise 0.
    ; --------------------------------------
    cpufeat MACRO feature:REQ
        push  ebx
        IFIDNI <feature>, <CPUID>
            ;; ---------------------------------
            ;; CPUID supported if can set/clear
            ;; ID flag (EFLAGS bit 21).
            ;; ---------------------------------
            pushfd
            pop   edx
            pushfd
            pop   eax
            xor   eax, 200000h  ; flip ID flag
            push  eax
            popfd
            pushfd
            pop   eax
            xor   eax, edx
        ELSEIFIDNI <feature>, <FPU>
            _cpuid 1
            mov   eax, brval(edx,0,0)
        ELSEIFIDNI <feature>, <TSC>
            _cpuid 1
            mov   eax, brval(edx,4,4)
        ELSEIFIDNI <feature>, <CX8>
            _cpuid 1
            mov   eax, brval(edx,8,8)
        ELSEIFIDNI <feature>, <CMOV>
            _cpuid 1
            mov   eax, brval(edx,15,15)
        ELSEIFIDNI <feature>, <CLFSH>
            _cpuid 1
            mov   eax, brval(edx,19,19)
        ELSEIFIDNI <feature>, <MMX>
            _cpuid 1
            mov   eax, brval(edx,23,23)
        ELSEIFIDNI <feature>, <FXSR>
            _cpuid 1
            mov   eax, brval(edx,24,24)
        ELSEIFIDNI <feature>, <SSE>
            _cpuid 1
            mov   eax, brval(edx,25,25)
        ELSEIFIDNI <feature>, <SSE2>
            _cpuid 1
            mov   eax, brval(edx,26,26)
        ELSEIFIDNI <feature>, <HTT>
            _cpuid 1
            mov   eax, brval(edx,28,28)
        ELSEIFIDNI <feature>, <SSE3>
            _cpuid 1
            mov   eax, brval(ecx,0,0)
        ENDIF        
        pop   ebx
        EXITM <eax>
    ENDM

    VENDOR_OTHER  EQU 0
    VENDOR_INTEL  EQU 1
    VENDOR_AMD    EQU 2

    NL EQU <13,10>


IdentifyCPU proc uses ebx esi
    LOCAL maxCpuidInput   :DWORD  ; maximum basic function
    LOCAL signature       :DWORD  ; EAX (function 1)
    LOCAL steppingID      :DWORD  ; EAX bits 3-0 (function 1)
    LOCAL model           :DWORD  ; EAX bits 7-4 (function 1)
    LOCAL displayModel    :DWORD  ; calculated
    LOCAL family          :DWORD  ; EAX bits 11-8 (function 1)
    LOCAL displayFamily   :DWORD  ; calculated
    LOCAL extendedModel   :DWORD  ; EAX bits 19-16 (function 1)
    LOCAL extendedFamily  :DWORD  ; EAX bits 27-20 (function 1)
    LOCAL brandID         :DWORD  ; EBX bits 7-0 (function 1)

    LOCAL maxCpuidInputEx :DWORD  ; maximum extended function
    LOCAL signatureEx     :DWORD  ; EAX (function 80000001h)

    LOCAL vendor          :DWORD  ; vendor identifier
    LOCAL vendorID[13]    :BYTE   ; (function 0)

    LOCAL nameString[48]  :BYTE   ; (function 80000002h-4h, AMD)

    mov   vendorID[12],   0       ; as a precaution
    mov   nameString[47], 0       ; as a precaution

    .IF cpufeat(CPUID) == 0
        printxa "CPUID not supported",crlf
        jmp   fini
    .ENDIF
    _cpuid 0
    mov   maxCpuidInput,          eax
    mov   DWORD PTR vendorID,     ebx
    mov   DWORD PTR vendorID[4],  edx
    mov   DWORD PTR vendorID[8],  ecx

    .if ebx=="uneG" && edx=="Ieni" && ecx=="letn"     ;"GenuineIntel"   
          mov   vendor, VENDOR_INTEL
    .elseif ebx=="htuA" && edx=="itne" && ecx=="DMAc" ; "AuthenticAMD"
          mov   vendor, VENDOR_AMD
    .else
          mov   vendor, VENDOR_OTHER
    .endif

    .IF maxCpuidInput > 0
        _cpuid 1
        mov   esi, eax
        mov   signature,      esi
        mov   steppingID,     brval(esi,3,0)
        mov   model,          brval(esi,7,4)
        mov   family,         brval(esi,11,8)
        mov   extendedModel,  brval(esi,19,16)
        mov   extendedFamily, brval(esi,27,20)
        mov   brandID,        brval(ebx,7,0)

        ; -----------------------------------------------
        ; For AMD only use the family and extendedFamily
        ; returned by function 80000001h, if available.
        ; -----------------------------------------------
        .IF vendor == VENDOR_AMD
            _cpuid 80000000h
            .IF eax > 80000000h
                _cpuid 80000001h
                mov   esi, eax
                mov   family,         brval(esi,11,8)
                mov   extendedFamily, brval(esi,27,20)
            .ENDIF
        .ENDIF
    .ENDIF

    ; -------------------------------------------------------
    ; The AMD Processor Name String, supported starting with
    ; the K5 Model 1, contains the actual processor name.
    ; The Intel Processor Brand String is retrieved in the
    ; same manner as the AMD Processor Name String, but
    ; instead of a processor name it contains frequency and
    ; multiplier information that is of no use here.
    ; -------------------------------------------------------
    .IF vendor == VENDOR_AMD
        _cpuid 80000000h
        mov   maxCpuidInputEx, eax
        .IF eax > 80000000h
            _cpuid 80000001h
            mov   signatureEx, eax
            .IF maxCpuidInputEx >= 80000004h
                _cpuid 80000002h
                mov   DWORD PTR nameString,     eax
                mov   DWORD PTR nameString[4],  ebx
                mov   DWORD PTR nameString[8],  ecx
                mov   DWORD PTR nameString[12], edx
                _cpuid 80000003h
                mov   DWORD PTR nameString[16], eax
                mov   DWORD PTR nameString[20], ebx
                mov   DWORD PTR nameString[24], ecx
                mov   DWORD PTR nameString[28], edx
                _cpuid 80000004h
                mov   DWORD PTR nameString[32], eax
                mov   DWORD PTR nameString[36], ebx
                mov   DWORD PTR nameString[40], ecx
                mov   DWORD PTR nameString[44], edx
            .ENDIF
        .ENDIF
    .ENDIF

    ; ------------------------------------------------------------
    ; Use the Intel rules to calc the displayed family and model.
    ; ------------------------------------------------------------
    mov eax,family
    mov displayFamily,eax
    mov eax,model
    mov displayModel,eax
    .IF family == 0Fh
        mov   eax, extendedFamily
        add   displayFamily, eax
    .ENDIF
    .IF family == 6 || family == 0Fh
        mov   eax, extendedModel
        add   displayModel, eax
    .ENDIF

    ; -----------------------------------------------------------
    ; Display the family, model, and stepping in decimal instead
    ; of the hex that Intel now recommends.
    ; -----------------------------------------------------------
    printxa addr vendorID,"  Family ",dd, displayFamily,"  Model ",dd,displayModel
    printxa "  Stepping ",dd,steppingID,crlf

    ; ----------------------------------------------------------
    ; For Intel processors, display the Intel Brand String from
    ; a table. If the brand ID is zero, display a processor
    ; identification based on the family, model, and features.
    ;
    ; The table is adapted from the IA-32 Intel Architecture
    ; Software Developers Manual Volume 2A, 253666-015,
    ; April 2005, Table 3-19.
    ; ---------------------------------------------------------
    .IF vendor == VENDOR_INTEL
        printxa "Intel Brand String: "
        mov eax, brandID
            .if eax==0
                printxa "NA"
                mov eax,family
                    .if eax== 4
                        printxa ", processor is 486"
                    .elseif eax== 5
                        .IF cpufeat(MMX)
                            printxa ", processor is Pentium MMX"
                        .ELSE
                            printxa ", processor is Pentium I"
                        .ENDIF
                    .elseif eax== 6
                        .IF cpufeat(SSE)
                            printxa ", processor is Pentium III"
                        .ELSEIF cpufeat(MMX)
                            printxa ", processor is Pentium II"
                        .ELSE
                            printxa ", processor is Pentium Pro"
                        .endif
                    .ENDIF
            .elseif eax== 1
                printxa "Intel(R) Celeron(R) processor"
            .elseif eax== 2
                printxa "Intel(R) Pentium(R) III processor"
            .elseif eax== 3
                .IF signature == 6B1h
                    printxa "Intel(R) Celeron(R) processor"
                .ELSE
                    printxa "Intel(R) Pentium(R) III Xeon(TM) processor"
                .ENDIF
            .elseif eax== 4
                printxa "Intel(R) Pentium(R) III processor"
            .elseif eax== 6
                printxa "Mobile Intel(R) Pentium(R) III processor-M"
            .elseif eax== 7
                printxa "Mobile Intel(R) Celeron(R) processor"
            .elseif eax== 8
                printxa "Intel(R) Pentium(R) 4 processor"
            .elseif eax== 9
                printxa "Intel(R) Pentium(R) 4 processor"
            .elseif eax== 0Ah
                printxa "Intel(R) Celeron(R) processor"
            .elseif eax== 0Bh
                .IF signature == 0F13h
                    printxa "Intel(R) Xeon(TM) processor MP"
                .ELSE
                    printxa "Intel(R) Xeon(TM) processor"
                .ENDIF
            .elseif eax== 0Ch
                printxa "Intel(R) Xeon(TM) processor MP"
            .elseif eax== 0Eh
                .IF signature == 0F13h
                    printxa "Intel(R) Xeon(TM) processor"
                .ELSE
                    printxa "Mobile Intel(R) Pentium(R) 4 processor-M"
                .ENDIF
            .elseif eax== 0Fh
                printxa "Mobile Intel(R) Celeron(R) processor"
            .elseif eax== 13h
                printxa "Mobile Intel(R) Celeron(R) processor"
            .elseif eax== 16h
                printxa "Intel(R) Pentium(R) M processor"
            .elseif eax== 17h
                printxa "Mobile Intel(R) Celeron(R) processor"
            .else
                printxa "NA, unknown Brand ID"
            .endif
            printxa crlf
    .ENDIF

    ; -----------------------------------------------------------
    ; For AMD processors, display the name string, if available,
    ; or a processor identification based on the family.
    ; -----------------------------------------------------------
    .IF vendor == VENDOR_AMD
        printxa "AMD Name String: "
        .IF maxCpuidInputEx >= 80000004h
            printxa addr nameString
        .ELSE
            .IF family == 4
                printxa "NA, processor is 486/5x86"
            .ELSE
                printxa "NA, processor is K5"
            .ENDIF
        .ENDIF
        printxa crlf
    .ENDIF    

    printxa "Features: "
    .IF cpufeat(FPU)
        printxa "FPU"
    .ENDIF
    .IF cpufeat(TSC)
        printxa " TSC"
    .ENDIF
    .IF cpufeat(CX8)
        printxa " CX8"
    .ENDIF
    .IF cpufeat(CMOV)
        printxa " CMOV"
    .ENDIF
    .IF cpufeat(CLFSH)
        printxa " CLFSH"
    .ENDIF
    .IF cpufeat(FXSR)
        printxa " FXSR"
    .ENDIF
    .IF cpufeat(HTT)
        printxa " HTT"
    .ENDIF    
    .IF cpufeat(MMX)
        printxa " MMX"
    .ENDIF
    .IF cpufeat(SSE)
        printxa " SSE"
    .ENDIF
    .IF cpufeat(SSE2)
        printxa " SSE2"
    .ENDIF
    .IF cpufeat(SSE3)
        printxa " SSE3"
    .ENDIF
    printxa " ",crlf

  fini:  
    ret
IdentifyCPU endp

