
;------------------------------------------------------------------------------------------
; ----------------- miscellaneous procs for setting up the timing test --------------------
;------------------------------------------------------------------------------------------

.data
CallTable dd MaxTests dup (0)       ; up to MaxTests tests allowed
TableEnd    equ $

ProgName    db 'Timing Test',0
.data?
align 4
CurProcess  dd ?
TestNum     dd ?
htemp       dd ?
;Loops      dd ?
hFnt        dd ?
headersize  dd ?
headers     db 256 dup (?)
.code

RunTest proc
    pusha
    inv Sleep,100
    inv UpdateWindow,hWin
    invoke GetCurrentProcess
    mov [CurProcess],eax
    invoke SetPriorityClass, CurProcess, REALTIME_PRIORITY_CLASS
    invoke SetProcessPriorityBoost, CurProcess, FALSE   ; normal behavior
    invoke SetThreadPriority, CurProcess, THREAD_PRIORITY_TIME_CRITICAL
    mov eax,TestNum
    mov eax,[CallTable+eax*4-4]
    .if eax
        call eax    ; call Test1 or Test2 or .... TestN
    .endif
    invoke SetThreadPriority, CurProcess, THREAD_PRIORITY_NORMAL
    invoke SetPriorityClass, CurProcess, NORMAL_PRIORITY_CLASS
    .if Overheadflg
        mov eax,TestNum
        add eax,5000        ; Result cycles
        inv SetDlgItemInt,hWin,eax,Result,TRUE
        if (tsttype eq DoMin) or (tsttype eq DoSep)
            Showit Min
        endif               
    .endif    
    .if PrintResults
        call PrintToFile
    .endif
    inv UpdateWindow,hWin
    popa
	ret
RunTest endp
DlgProc proc uses ebx, hWnd,uMsg,wParam,lParam
    .if uMsg == WM_COMMAND
        .if wParam > 2000 && wParam < 2999 ; one of the test buttons
            mov eax,wParam
            sub eax,2000
            mov TestNum,eax ; save the actual test number, 1..999
            call RunTest
        .elseif wParam==1101    ; run all tests
            mov TestNum,1
            mov ebx,offset CallTable
            .repeat
                .if dword ptr [ebx]   ; was there an address set up?
                    call RunTest
                .endif
                add ebx,4
                add TestNum,1
            .until TestNum>MaxTests
        .endif
        
    .elseif uMsg== WM_INITDIALOG
        pusha
        mov eax,hWnd    ; save handle for everyone
        mov hWin,eax
        inv SendMessage,hWin,WM_GETFONT,0,0 ; get the font for the controls to use
        mov hFnt,eax
        BeginCounter
          ; null test to get overhead
        EndCounter
        mov eax,Result
        mov Overhead,eax
        mov Overheadflg,1
        mov edx,Overhead
        inv SetDlgItemInt,hWin,1103,edx,TRUE
        inv SetDlgItemInt,hWin,1006,TimesToLoop,FALSE   ; print out number of times we are looping
        call CallTableSetup ; go set up the address for the procs to test, add controls, etc.
        printx offset headers
        call IdentifyCPU
        if tsttype eq DoMin
            printxa crlf,"Method: Min",crlf
        elseif tsttype eq DoAvg
            printx crlf,"Method: Average",crlf
        endif
        .listall
        printxa "Iterations: ",dd,TimesToLoop,crlf
        printxa '        Clocks   Description',crlf
        push printxsize
        pop headersize
        popa
    .elseif uMsg == WM_CLOSE
        invoke  EndDialog,hWin,0
    .endif

    xor eax,eax
    ret
DlgProc endp


.data
Wide1 = 50      ; width of Test Button
Wide2 = 212     ; width of Description
Wide3 = 75      ; width of Clock Cycles
Wide4 = 75      ; width of Average Clocks
Wide5 = 75      ; width of scratch
Left1 = 11      ; Button   2001, 2002, .... , 2020   
Left2 = Left1+Wide1+4       ; Edit 1, Description, will compute location, 3001, 3002, ... , 3020
Left3 = Left2+Wide2+2       ; Edit 2, Clock Cycles, 4001, ...
Left4 = Left3+Wide3+2       ; Edit 3, Average Clocks 5001, ...
Left5 = Left4+Wide4+2       ; Edit 4, Test routine scratch for output value if desired, 6001, ...
TDel  = 32      ; Delta between rows
Top  dd 120     ; top of all, will increment by TDel
Ht   dd TDel        ; height of rows
blank   db ' ',0
aBlank  dd offset blank
edt     db 'EDIT',0,0
CtlGp   dd  2000    ; base id

.code
MakeControls proc
    mov eax,2000        ; each starts with id 2000+TestNum
    add eax,TestNum
    mov CtlGp,eax
    inv CreateWindowEx,0,soff("BUTTON"),abuff,
            WS_CHILD or WS_VISIBLE or WS_TABSTOP,
            Left1,Top,Wide1,Ht,hWin,CtlGp,temp1,NULL
    inv SendMessage,eax,WM_SETFONT,hFnt,TRUE
    add CtlGp,1000
    
    inv CreateWindowEx,WS_EX_CLIENTEDGE,addr edt,aBlank,
            WS_VISIBLE or WS_CHILDWINDOW or ES_NOHIDESEL or ES_MULTILINE,
            Left2,Top,Wide2,Ht,hWin,CtlGp,temp1,NULL
    inv SendMessage,eax,WM_SETFONT,hFnt,TRUE
    add CtlGp,1000
    
    inv CreateWindowEx,WS_EX_CLIENTEDGE,addr edt,aBlank,
            WS_VISIBLE or WS_CHILDWINDOW or ES_AUTOHSCROLL or ES_NOHIDESEL,
            Left3,Top,Wide3,Ht,hWin,CtlGp,temp1,NULL
    inv SendMessage,eax,WM_SETFONT,hFnt,TRUE
    add CtlGp,1000
    
    inv CreateWindowEx,WS_EX_CLIENTEDGE,addr edt,aBlank,
            WS_VISIBLE or WS_CHILDWINDOW or ES_AUTOHSCROLL or ES_NOHIDESEL,
            Left4,Top,Wide4,Ht,hWin,CtlGp,temp1,NULL
    inv SendMessage,eax,WM_SETFONT,hFnt,TRUE
    add CtlGp,1000
    
    inv CreateWindowEx,WS_EX_CLIENTEDGE,addr edt,aBlank,
            WS_VISIBLE or WS_CHILDWINDOW or ES_AUTOHSCROLL or ES_NOHIDESEL,
            Left5,Top,Wide5,Ht,hWin,CtlGp,temp1,NULL
    inv SendMessage,eax,WM_SETFONT,hFnt,TRUE

    mov eax,TDel    ; set up for next group
    add Top,eax

    ret
MakeControls EndP

.data
ctsfmt db "Test %i",0
WinRect   RECT <?>  ; left,top,right,bottom of Dialog window
.code
CallTableSetup:  ; call table setup, save offsets of tests, and also add controls to dialog
    mov ebx,offset CallTable
    num=0
    Repeat MaxTests
        num=num+1
        arg CatStr <Test>, %num     ;; construct  Test1, Test2, etc.
        if (opattr(arg)) and 01y    ;; if it is a label
            mov TestNum,num
            mov [ebx],offset arg
            call DoSetup
        endif
        add ebx,4       ;; set up for next store address
    endm    

    ; set window height here
    inv GetWindowRect,hWin,addr WinRect
    mov eax,WinRect.right
    sub eax,WinRect.left        ;width of dialog
    mov WinRect.right,eax       ;save width
    mov eax,Top                 ; would be top of next row added
    mov WinRect.bottom,eax
    add WinRect.bottom,TDel     ;new height
    inv MoveWindow,hWin,WinRect.left,0,WinRect.right,WinRect.bottom,TRUE   
    ret

DoSetup:
            pusha   ;; save everything while we add controls, etc.
            inv wsprintf,abuff,addr ctsfmt,TestNum   ; get button caption
            call MakeControls
            mov eax,[ebx]           
            call eax        ; call for first time just to get description
            popa
            ret

SetDesc proc    ; enter with Description Offset in ebx
    pusha       ; save users setup if any
    push ebx    
    mov eax,TestNum
    add eax,3000        ; get the id of the description box
    GetHandle eax,htemp
    pop ebx     ; get back Description offset
    invoke SendMessage,htemp,WM_SETTEXT,0,ebx
    popa
    ret
SetDesc EndP
.data?
hf   dd ?  ; handle for result file
.data
.code

PrintToFile:
    pusha   ; time is not a factor here
    
    inv _lcreat,soff("Results.txt"),0
    mov hf,eax

    inv _lwrite,hf,addr headers,headersize
    push TestNum
    mov TestNum,1
    mov ebx,offset CallTable
    .repeat
        .if dword ptr [ebx]   ; was there an address set up?
            mov esi,TestNum
            call DoPrint
        .endif
        add ebx,4
        add TestNum,1
    .until TestNum>MaxTests
    pop TestNum

    inv CloseHandle,hf
    popa
    ret   

DoPrint:
        mov edi,abuff
        mov al,' '
        mov ecx,99
        rep stosb
        add esi,2000        ; each starts with id 2000+TestNum
        mov edi,abuff
        inv GetDlgItemText,hWin,esi,edi,100 ; get button caption
        mov [edi+eax],byte ptr ' '  ; remove zero byte terminator
        add edi,8
        add esi,3000
        inv GetDlgItemText,hWin,esi,edi,100 ; get clock ticks
        mov [edi+eax],byte ptr ' '  ; remove zero byte terminator
        .if dword ptr [edi]=='    '
            ret
        .endif
        add edi,9
        sub esi,2000        ; get desc
        inv GetDlgItemText,hWin,esi,edi,100
        add edi,eax
        mov [edi],word ptr 0a0dh
        add edi,2
        sub edi,abuff                        
        inv _lwrite,hf,abuff,edi
        ret

dwtoax2:        ; adapted from what masm32 lib's dwtoa
                ; convert eax, save at address in edi, return with edi pointing to zero byte terminator
                ; also saves ALL registers (except edi which is updated) 
    or eax,eax  ; test for zero and negative values
    .if Zero?
        mov word ptr [edi],30h  ; "0" and zero byte terminator
        inc edi     ; point at terminator
        ret
    .endif
    push esi
    push ecx
    push edx
    push ebx
    .if Sign?    
        mov byte ptr [edi],'-'
        neg eax
        add edi, 1
    .endif
    mov ecx, 0CCCCCCCDh ; 8 * 1/10
    mov esi, edi    ; save start of string
    .repeat
        mov ebx,eax
        mul ecx
        shr edx, 3
        mov eax,edx
        lea edx,[edx*4+edx]
        add edx,edx
        sub ebx,edx
        add bl,'0'
        mov [edi],bl
        add edi, 1
    .until eax==0
    mov byte ptr [edi], 0   ; terminate the string
    mov ecx,edi
    .repeat                 ; We now have all the digits, but in reverse order.
      sub ecx, 1            ; swap as needed
      mov al, [esi]
      mov ah, [ecx]
      mov [ecx], al
      mov [esi], ah
      add esi, 1
    .until (esi >= ecx)
    pop ebx
    pop edx
    pop ecx
    pop esi
    ret

include idcpux.inc

end Program
