;--------------------- printx macros -------------------------
;      copies multiple strings to a buffer, optionally converting dwords
;      to text using dd keyword, or byte directly if immediate value < 256
.data
printxname  db "printx",0
printxstart dd 0    ; start of output buffer as defined by printx or printxf
printxptr   dd 0    ; next available spot for appending more text
printxsize  dd 0    ; current number of characters in the buffer
printxerr1  db "Error in printx: invalid input address",0
printxerr2  db "printx called without setting start address",0
printxerr3  db "Error in printx: invalid output address",0
printxeflg  db 0
.code
align 4
printxproc: ; proc ; the main printx procedure called by the printx macro
    ; all registers are saved by printxa macro
    ; edi = output address
    ; dh  = flag, 0=string, address in esi
    ;             1=signed dword in eax, convert to ascii
    ;             2=byte (no conversion, value in al)
    ;             3=unsigned dword in eax, convert to ascii
    mov edi,printxptr    ;; where we will store the results
    .if edi ; has the user set an address?
        .if dh==1 || dh==3 ; flag to convert dword to text
            ; adapted from masm32 lib's dwtoa but can convert to both signed and unsigned values
            or eax,eax  ; test for zero and negative values
            .if Zero?
                mov word ptr [edi],30h  ; "0" and zero byte terminator
                inc edi     ; point at terminator
            .else
                .if Sign?    
                    .if dh==1    ; check signed
                        mov byte ptr [edi],'-'
                        neg eax
                        add edi, 1
                    .endif
                .endif
                mov ecx, 0CCCCCCCDh ; 8 * 1/10
                mov esi, edi    ; save start of string
                .repeat
                    mov ebx,eax
                    mul ecx
                    shr edx, 3
                    mov eax,edx
                    lea edx,[edx*4+edx]
                    add edx,edx
                    sub ebx,edx
                    add bl,'0'
                    mov [edi],bl
                    add edi, 1
                .until eax==0
                mov byte ptr [edi], 0   ; terminate the string
                mov ecx,edi
                .repeat                 ; We now have all the digits, but in reverse order.
                  sub ecx, 1            ; swap as needed
                  mov al, [esi]
                  mov ah, [ecx]
                  mov [ecx], al
                  mov [esi], ah
                  add esi, 1
                .until (esi >= ecx)
            .endif
        .elseif dh==2  ; flag to save a byte
            stosb
            mov byte ptr [edi],0    
        .else
            mov printxeflg,0 ;Install per-thread error handler from http://www.winasm.net/forum/index.php?showtopic=2082
            assume fs:nothing ;MASM assumes the use of this register to be ERROR by default
            push offset PrintxExceptionHandler
            push fs:[0]
            mov fs:[0],esp
            .repeat
                lodsb
                stosb
            .until al==0
            dec edi
            PrintxErrRet:
            pop fs:[0]   ; Uninstall per-thread error handler
            add esp,4
            .if printxeflg
                mov eax,offset printxerr1
                jmp printxerrorexit
            .endif
        .endif
        mov printxptr,edi
        sub edi,printxstart
        mov printxsize,edi
    .else
        mov eax,offset printxerr2
        printxerrorexit:
        invoke MessageBox,0,eax,addr printxname,0
    .endif
	ret

PrintxExceptionHandler: ; PROC pExcept, pFrame, pContext, pDispatch
    mov eax,[esp+12]  ; pContext
    mov [eax].CONTEXT.regEip,offset PrintxErrRet
    mov eax,ExceptionContinueExecution
    mov printxeflg,1
    ret

CheckPrintxAddress:
    pusha
    mov printxptr,eax
    mov printxstart,eax
    mov printxsize,0

    mov printxeflg,0 ;Install per-thread error handler from http://www.winasm.net/forum/index.php?showtopic=2082
    assume fs:nothing ;MASM assumes the use of this register to be ERROR by default
    push offset CheckPrintxAddressExceptionHandler
    push fs:[0]
    mov fs:[0],esp
    
    mov dword ptr [eax],0

    CheckPrintxAddressErrRet:
    pop fs:[0]   ; Uninstall per-thread error handler
    add esp,4
    .if printxeflg
        mov eax,offset printxerr3
        invoke MessageBox,0,eax,addr printxname,0
        mov printxptr,0     ; don't use this address!
        mov printxstart,0
    .endif
    popa
    ret

CheckPrintxAddressExceptionHandler:
    mov eax,[esp+12]  ; pContext
    mov [eax].CONTEXT.regEip,offset CheckPrintxAddressErrRet
    mov eax,ExceptionContinueExecution
    mov printxeflg,1
    ret

printx macro BufferAddr,Things:Vararg ;; copy all Things to BufferAddr
  ;; copys text strings surrounded by double or single quotes to buffer BufferAddr
  ;; an entry of the characters dd means the next value is a dword to be converted to text
  ;; printxa (printx append) may be called to append more text to current buffer
  ;; printxf is the function version of printx, which returns BufferAddr
  ;; printxaf is the function version of printxa, append to buffer, returns BufferAddr  
    push eax
    if @SizeStr(BufferAddr) gt 5
        ifidni @SubStr(BufferAddr,1,5),<addr >
            printxbuffaddr textequ @SubStr(BufferAddr,6)
            if (opattr(printxbuffaddr)) eq 98
                lea eax,printxbuffaddr
            else
                mov eax,@CatStr(<offset >,printxbuffaddr)
            endif
        elseif @SizeStr(BufferAddr) gt 7
            ifidni @SubStr(BufferAddr,1,7),<offset >
                printxbuffaddr textequ @SubStr(BufferAddr,8)
                if (opattr(printxbuffaddr)) eq 98
                    lea eax,printxbuffaddr
                else
                    mov eax,BufferAddr
                endif
            else
                mov eax,BufferAddr
            endif
        else
            mov eax,BufferAddr
        endif
    else
        mov eax,BufferAddr        
    endif
    call CheckPrintxAddress ; go see if ok, and save if so
    pop eax
    .if printxstart
        printxa Things  ;; go append to buffer
    .endif   ; else error with buffer address 
endm
printxf macro BufferAddr,Things:Vararg ;; printx function returns buffer address
    printx BufferAddr,Things
    exitm <printxstart>
endm
printxaf macro ThingsToPrint:VARARG ;; printx append function, returns buffer address
    printxa ThingsToPrint
    exitm <printxstart>
endm
printxa macro ThingsToPrint:VARARG  ;; printx appends text to buffer starting at printxptr
    ddflg=0
    for arg,<ThingsToPrint>
        xxxxxxxxxxxxxxxxxxxxxxxx textequ <&arg> ;; for debug
        if @SizeStr(arg) eq 0
            goto DoneArg
        endif            
        ifIDNi <arg>,<dd>  ;; is the next value a dword?
            ddflg=1
            goto DoneArg
        elseifIDNi <arg>,<du>  ;; is the next value an unsigned dword?
            ddflg=3
            goto DoneArg
        endif            
        pusha ;; save any registers that may be printed from later in this call
        if (@InStr(,<!"!'>,@SubStr(arg,1,1)))   ;; if single or double quote
            .data
               tmpoffset = $
               db arg,0
            .code
            mov esi,tmpoffset
            goto DoZero
        endif
        if ddflg
;;echo doing arg        
            opxttr = opattr(arg)
;;opxx textequ %opxttr
;;%echo opp= opxx &arg
            if opxttr eq 110000y ;; 30h a register
                if @SizeStr(arg) eq 2
                    movzx eax,arg
                else
                    mov eax,arg
                endif
            elseif opxttr eq 100100y ;; 24h immediate value
               mov eax,arg
            elseif opxttr eq 101010y or opxttr eq 100010y ;; 22h or 2Ah a variable name
                typex = type arg
                if typex eq 2
                    movzx eax,word ptr arg
                elseif typex eq 1
                    movzx eax,byte ptr arg
                else
                    mov eax,arg
                endif
            else                
;;opxx textequ %opxttr
;;%echo opp= opxx &arg
                mov eax,arg  ;; will produce an invalid address error
            endif

            if ddflg eq 1                
                goto DoOne
            else
                goto DoThree
            endif
        endif
        if @SizeStr(arg) gt 5
            ifidni @SubStr(arg,1,5),<addr >
                lea esi,@SubStr(arg,6)
                goto DoZero
            endif
            ifidni @SubStr(arg,1,5),<aval >
                mov esi,@SubStr(arg,6)
                goto DoZero
            endif
            ;; else fall out to DoNormal
        endif          
        if (opattr(arg)) eq 100100y   ;; immediate value
            if (arg gt 0) and (arg lt 256)
                mov al,arg
                mov dh,2
                goto Doit
            endif
            mov eax,arg     ;; just print a large value
            goto DoOne      ;; default to signed
        endif
      :DoNormal
        if (opattr(arg)) eq 98
            mov esi,arg
            goto DoZero
        endif
        lea esi,[arg]
      :DoZero
        mov dh,0
        goto Doit
      :DoThree
        mov dh,3
        goto Doit
      :DoOne
        mov dh,1
      :Doit  
        call printxproc
        ddflg=0
        popa
     :DoneArg
    endm
endm
msgx macro args:vararg ;; mostly for testing printx
    pusha
    ifnb <args>
        printx args
    endif
    mov eax,printxstart
    .if printxstart
        ifdef ProgName
            invoke MessageBox,0,printxstart,addr ProgName,0
        else
            invoke MessageBox,0,printxstart,0,0
        endif
    .endif
    popa
endm
ifndef m2m
m2m MACRO M1:req, M2:req
	push M2
	pop  M1
ENDM
endif
